# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the control panel's Qt main."""

from twisted.internet.defer import inlineCallbacks

from ubuntuone.controlpanel.gui.qt import main
from ubuntuone.controlpanel.tests import TestCase


class FakeApplication(object):

    """A fake application."""

    def __init__(self):
        self.args = None
        self.style = None

    def __call__(self, argv, *args, **kwargs):
        """Fake arg filtering function."""
        if '-title' in argv:
            argv.remove('-title')
        self.args = (argv, args, kwargs)
        return self

    # pylint: disable=C0103
    def setStyleSheet(self, *args, **kwargs):
        """Fake setStyleSheet."""
        self.style = (args, kwargs)
    # pylint: enable=C0103

    def arguments(self):
        """Fake arguments function."""
        return self.args[0]

    def exec_(self):
        """Fake event loop."""
        pass


class FakeMainWindow(object):

    """A fake MainWindow."""

    tabname = None
    urgent = None

    def switch_to(self, tabname):
        """Fake switch_to."""
        self.tabname = tabname

    def set_urgent(self, value):
        """Fake set_urgent."""
        self.urgent = value


class FakeStart(object):

    """Fake start function."""

    def __init__(self):
        self.args = None
        self.window = None

    def __call__(self, *args, **kwargs):
        self.args = (args, kwargs)
        self.window = FakeMainWindow()
        return None, self.window


class MainTestCase(TestCase):

    """Test the argument parsing and passing in main."""

    @inlineCallbacks
    def setUp(self):
        yield super(MainTestCase, self).setUp()
        self.app = FakeApplication()
        self.start = FakeStart()
        self.patch(main, "UniqueApplication", self.app)
        self.patch(main, "start", self.start)

    def test_wm_class(self):
        """Test that we set the 1st argument, used for WM_CLASS, correctly."""
        main.main([])
        self.assertEqual(self.app.args,
            (['ubuntuone-installer'], ('ubuntuone-control-panel',), {}))

    def test_title_not_fail(self):
        """Ensure -title is removed before it gets to OptParser."""
        main.main(["blah", "-title"])
        # Did not crash!

    def test_minimized_option(self):
        """Ensure the --minimized option is parsed and passed correctly."""
        main.main(["blah", "--minimized"])
        self.assertEqual(self.start.args[1],
            {'minimized': True, 'with_icon': False})

    def test_with_icon_option(self):
        """Ensure the --minimized option is parsed and passed correctly."""
        main.main(["blah", "--with-icon"])
        self.assertEqual(self.start.args[1],
            {'minimized': False, 'with_icon': True})

    def test_style_loads(self):
        """Ensure the stylesheet is loaded."""
        main.main([])
        self.assertTrue(self.app.style)

    def test_switch_to_option(self):
        """Ensure the --switch-to option is parsed and passed correctly."""
        main.main(["foobar", "--switch-to", "folders"])
        self.assertEqual(self.start.window.tabname, "folders")
