# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the First-time wizard."""

from twisted.internet import defer

from ubuntuone.controlpanel.gui.qt import wizard as gui
from ubuntuone.controlpanel.gui.qt.tests import BaseTestCase, TOKEN


class UbuntuOneWizardTestCase(BaseTestCase):
    """Test the UbuntuOneWizard."""

    class_ui = gui.UbuntuOneWizard

    def test_options(self):
        """Tne wizard options are correct."""
        options = [
            (self.ui.NoBackButtonOnStartPage, True),
            (self.ui.HaveFinishButtonOnEarlyPages, False),
        ]
        for option, expected in options:
            self.assertEqual(expected, self.ui.testOption(option))

    def test_style(self):
        """Tne wizard style is Modern."""
        self.assertEqual(self.ui.wizardStyle(), self.ui.ModernStyle)

    def test_cancel_button(self):
        """Send the rejected signal when the cancel button is clicked."""
        button = self.ui.button(self.ui.CancelButton)
        self.assertEqual(button.text(), gui.CLOSE_AND_SETUP_LATER)

        self.ui.rejected.connect(self._set_called)
        button.click()

        self.assertEqual(self._called, ((), {}))

    def test_button_layout(self):
        """The button layout is correct."""
        buttons = [
            'BackButton', 'CommitButton', 'CustomButton1', 'CustomButton2',
            'CustomButton3', 'FinishButton', 'HelpButton', 'NextButton',
        ]
        for button_name in buttons:
            button = self.ui.button(getattr(self.ui, button_name))
            self.assertFalse(button.isVisible(),
                             'Button %s should not be visible.' % button_name)

        button = self.ui.button(self.ui.CancelButton)
        self.assertTrue(button.isVisible(),
                        'Cancel button should not be visible.')

    def test_first_page(self):
        """The first page is the correct one."""
        self.assertEqual(self.ui.startId(),
                         self.ui.pages[self.ui.signin_page])

    def test_tab_order(self):
        """The button tab order is correct."""
        # can not be tested due to Qt API limitations


class SideWidgetTestCase(UbuntuOneWizardTestCase):
    """Test the side widget in the wizard."""

    def test_is_there(self):
        """The side widget is correct."""
        self.assertIsInstance(self.ui.side_widget, gui.SideWidget)
        self.assertIs(self.ui.sideWidget(), self.ui.side_widget)


class SignInPageTestCase(UbuntuOneWizardTestCase):
    """Test the SignInPage wizard page."""

    page_name = 'signin'

    @defer.inlineCallbacks
    def setUp(self):
        yield super(SignInPageTestCase, self).setUp()
        self.page = getattr(self.ui, '%s_page' % self.page_name)

    def test_was_added(self):
        """The SignInPage is added correctly."""
        page_id = self.ui.pages[self.page]
        self.assertIs(self.ui.page(page_id), self.page)

    def test_enabled_at_startup(self):
        """The page is enabled at startup."""
        self.assertTrue(self.page.isEnabled())


class LoginTestCase(UbuntuOneWizardTestCase):
    """Test the login through the wizard."""

    method = 'login'

    @defer.inlineCallbacks
    def test_with_credentials(self):
        """Wizard is done when credentials were retrieved."""
        self.ui.finished.connect(self._set_called)
        d = defer.succeed(TOKEN)

        def check():
            """Confirm the ui is disabled while processing."""
            self.assertFalse(self.ui.signin_page.isEnabled())
            return d

        self.patch(self.ui.backend, self.method, check)
        button = getattr(self.ui.signin_page.panel.ui,
                         '%s_button' % self.method)
        button.click()

        yield d

        self.assertTrue(self.ui.signin_page.isEnabled())
        self.assertEqual(self._called, ((1,), {}))

    @defer.inlineCallbacks
    def test_without_credentials(self):
        """Wizard is done when credentials were retrieved."""
        self.ui.finished.connect(self._set_called)
        d = defer.succeed(None)

        def check():
            """Confirm the ui is disabled while processing."""
            self.assertFalse(self.ui.signin_page.isEnabled())
            return d

        self.patch(self.ui.backend, self.method, check)
        button = getattr(self.ui.signin_page.panel.ui,
                         '%s_button' % self.method)
        button.click()

        yield d

        self.assertTrue(self.ui.signin_page.isEnabled())
        self.assertFalse(self._called)


class RegisterTestCase(LoginTestCase):
    """Test the register through the wizard."""

    method = 'register'
