# -*- coding: utf-8 -*-

#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test the windows utils functions."""

import os

from twisted.internet import defer

from ubuntuone.controlpanel import utils
from ubuntuone.controlpanel.tests import TestCase

# let me use protected methods
# pylint: disable=W0212


class AutoupdaterTestCase(TestCase):
    """Test the code that is used for the auto update process."""

    @defer.inlineCallbacks
    def setUp(self):
        """Prepare for the diff tests."""
        yield super(AutoupdaterTestCase, self).setUp()
        self.auto_update_path = r'path\to\exe'
        self.return_from_call = 0
        self.command = None
        self.args = []

        def fake_execute_process(command, args=None, path=None):
            """Fake async process execution."""
            self.command = command
            self.args = args
            return self.return_from_call

        self.patch(utils.windows, 'getProcessValue', fake_execute_process)
        self.patch(utils.windows, '_get_update_path',
                   lambda: self.auto_update_path)

    @defer.inlineCallbacks
    def test_are_updates_present_true(self):
        """Test when updates are present."""
        # the idea is simple, set the value to be returned from
        # the fake call, assert that we get true and also that
        # we did use the correct parameters.
        self.return_from_call = 0
        are_present = yield utils.are_updates_present()
        self.assertTrue(are_present, 'Updates should be present.')
        # lets assert that we did use the correct args
        expected_args = ('--mode', 'unattended')
        self.assertEqual(expected_args, self.args)
        self.assertEqual(self.command, self.auto_update_path)

    @defer.inlineCallbacks
    def test_are_updates_present_false(self):
        """Test when updates are not present."""
        # similar to test_are_updates_present_true but with diff retcode
        self.return_from_call = 40
        are_present = yield utils.are_updates_present()
        self.assertFalse(are_present, 'Updates should NOT be present.')
        # lets assert that we did use the correct args
        expected_args = ('--mode', 'unattended')
        self.assertEqual(expected_args, self.args)
        self.assertEqual(self.command, self.auto_update_path)

    def test_perform_update(self):
        """Test the method that performs the update."""
        self.patch(utils.windows.win32api, 'ShellExecute', self._set_called)
        utils.perform_update()
        self.assertIn(self.auto_update_path, self._called[0][2])
        self.assertEqual('runas', self._called[0][1])
        self.assertEqual('--unattendedmodeui none', self._called[0][3])
        self.assertEqual('', self._called[0][4])
        self.assertEqual(0, self._called[0][5])


class GetPathTestCase(TestCase):
    """Test the code that is used for the auto update process."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set the different tests."""
        yield super(GetPathTestCase, self).setUp()
        self.called = []
        self.exec_path = 'path/to/current/exe'
        self.exists = True

        def fake_abspath(path):
            """Fake os.path.abspath."""
            self.called.append('os.path.abspath')
            return path

        def fake_dirname(path):
            """Fake os.path.dirname."""
            self.called.append('os.path.dirname')
            return path

        def fake_exists(path):
            """Fake os.path.exists."""
            self.called.append('os.path.exists')
            return self.exists

        # patch the os.path functions used
        self.patch(utils.windows.os.path, 'abspath', fake_abspath)
        self.patch(utils.windows.os.path, 'dirname', fake_dirname)
        self.patch(utils.windows.os.path, 'exists', fake_exists)

    def _delete_frozen_state(self):
        """Delete the frozen state."""
        del utils.windows.sys.frozen
        del utils.windows.sys.executable

    def test_get_auto_update_path_frozen(self):
        """Test the method used to get the autoupdate."""
        # patch the diff parts of sys so that we get fake paths
        is_frozen = hasattr(utils.windows.sys, 'frozen')
        if not is_frozen:
            utils.windows.sys.frozen = True
            utils.windows.sys.executable = self.exec_path
            self.addCleanup(self._delete_frozen_state)

        # called method and assert that we have the correct result
        path = utils.windows._get_update_path()
        self.assertEqual(os.path.join(self.exec_path,
                         utils.windows.AUTOUPDATE_EXE_NAME), path)
        self.assertTrue('os.path.abspath' in self.called)
        self.assertTrue('os.path.dirname' in self.called)
        self.assertTrue('os.path.exists' in self.called)

    def _reset_frozen_state(self, old_frozen, old_exec_path):
        """Reset the frozen state."""
        utils.windows.sys.frozen = old_frozen
        utils.windows.sys.executable = old_exec_path

    def _reset__file__(self, path):
        """Reset the value of __file__."""
        utils.windows.__file__ = path

    def test_get_auto_update_path_not_frozen(self):
        """Test the method used to get the autoupdate."""
        is_frozen = hasattr(utils.windows.sys, 'frozen')
        if is_frozen:
            old_frozen = utils.windows.sys.frozen
            old_exec_path = utils.windows.sys.executable
            del utils.windows.sys.frozen
            del utils.windows.sys.executable
            self.addCleanup(self._reset_frozen_state, old_frozen,
                            old_exec_path)
        # set a fake __file__ for the module
        old_file = utils.windows.__file__
        utils.windows.__file__ = self.exec_path
        self.addCleanup(self._reset__file__, old_file)

        path = utils.windows._get_update_path()
        self.assertEqual(os.path.join(self.exec_path,
                         utils.windows.AUTOUPDATE_EXE_NAME), path)
        self.assertEqual(2, self.called.count('os.path.dirname'))
        self.assertTrue('os.path.exists' in self.called)

    def test_get_auto_update_path_not_present(self):
        """Test the method used to get the autoupdate."""
        self.exists = False

        # called method and assert that we have the correct result
        path = utils.windows._get_update_path()
        self.assertEqual(None, path)
